﻿define("fotowareintegration/editors/FotoWareThumbnailSelector", [
    // dojo
    "dojo/_base/declare",
    "dojo/when",
    "dojo/on",
    "dojo/topic",
    "dojo/dom-class",
    "dojo/ready",
    "dojo/_base/lang",
    // dijit
    "dijit/focus",
    "dijit/ProgressBar",
    // epi
    "epi/Url",
    "epi/routes",
    "epi/shell/dnd/Target",
    "epi/shell/DialogService",
    "epi/shell/XhrWrapper",
    // epi-cms
    "epi-cms/core/ContentReference",
    "epi-cms/widget/MediaSelector",
    "epi-cms/widget/ThumbnailSelector",
    "epi-cms/command/UploadContent",
    "epi-cms/widget/UploadUtil",
    "epi-cms/widget/viewmodel/MultipleFileUploadViewModel",
    "epi-cms/widget/LocalFolderUploader",
    "dojo/text!./templates/FotoWareThumbnailSelector.html",
    "epi/i18n!epi/cms/nls/episerver.cms.widget.thumbnailselector"
], function (
    // dojo
    declare,
    when,
    on,
    topic,
    domClass,
    ready,
    lang,
    // dijit
    focusManager,
    ProgressBar,
    // epi
    Url,
    routes,
    Target,
    dialogService,
    XhrWrapper,
    // epi-cms
    ContentReference,
    MediaSelector,
    ThumbnailSelector,
    UploadContent,
    UploadUtil,
    MultipleFileUploadViewModel,
    LocalFolderUploader,
    template,
    resources
) {
        var xhr = new XhrWrapper();
        return declare([ThumbnailSelector], {
            // summary:
            //      Represents the widget to select ContentReference.
            // tags:
            //      internal

            resources: resources,
            templateString: template,

            loaded: false,
            uploadDisabledMessage: "Drag and drop feature has been disabled, please use Fotoware.",
            fotowarepath: null,
            fotowareiconpath: null,
            fotowaretenanturl: null,
            fotowaremodulepath: null,
            fotowarepluginWindow: null,
            fwWin: null,
            pluginurl: null,
            timer: null,
            disableMediaSelector: null,
            listenerBound: false,
            //allowedExtensions: [public] Array
            //      List of file extensions that may be dropped onto the editor
            allowedExtensions: [],
            postCreate: function () {

                ready(setTimeout(lang.hitch(this, function () {//not optimal with timeout, but else not working with _setValue when used on init
                    this.loaded = true;
                }), 100));

                this.allowedExtensions = this.fotowareallowedfiletypes.replace(".", "").replace(/,/gi, "").split('.');

                this.inherited(arguments);
            },
            getFetchCf: function (url, data) {
                var self = this;
                return xhr.xhrGet({
                    url: url,
                    content: data,
                    failOk: true,
                    error: function (response) {
                        //console.log(self, response.responseText);
                        self._dialogService.alert(response.responseText);
                    }
                });
            },
            popupWinCenter: function (url, title, w, h) {
                var left = (screen.width / 2) - (w / 2);
                var top = (screen.height / 2) - (h / 2);

                var newWindow = window.open(url, title, 'resizable=yes, scrollbars, width=' + w + ', height=' + h + ', top=' + top + ', left=' + left);

                // Puts focus on the newWindow
                if (window.focus) newWindow.focus();

                return newWindow;
            },
            // Custom function for Fotoware integration to trigger opening of dialog.
            _onFotoWareButtonClick: function () {

                this.pluginurl = this.fotowarepath + "ClientResources/plugin/fotoware.html?fotowareBaseUrl=" +
                    this.fotowaretenanturl +
                    "/fotoweb/widgets/selection&insertMultiple=false&cn=" + this.fotowarecookiename + "&";

                if (window.fotowarepluginWindow) {//check if we have other windows open, close them, they wont work.
                    if (window.fotowarepluginWindow.name !== this.name) {
                        window.fotowarepluginWindow.close();
                    }
                }

                this.fwWin = this.popupWinCenter(this.pluginurl + Math.random(), this.name, 1200, 800);

                if (!this.listenerBound) {
                    this._listener = this._listener.bind(this);
                    window.fotowarepluginWindow = this.fwWin;
                    window.addEventListener("message", this._listener, true);
                    this.listenerBound = true;
                }

                this.timer = setInterval(lang.hitch(this,
                    function () {
                        if (this.fwWin.closed) {
                            window.removeEventListener("message", this._listener, true);
                            this.listenerBound = false;
                            clearInterval(this.timer);
                            return;
                        }
                    }),
                    2000);

            },
            _listener: function (event) {
                if (this.name === event.source.name) {//check if message is for this property
                    //ta hand om loggedinSuccess

                    
                    if (event.data == "loggedinSuccess") {
                        this.fwWin.location = this.pluginurl + Math.random();
                        return;
                    }
                    else if (event.data.event === 'selectionWidgetCancel') {
                        this.fwWin.close();
                        //remove listener
                        window.removeEventListener("message", this._listener, true);
                        this.listenerBound = false;
                    }
                    else if (event.data.event === 'assetSelected') {
                        if (this.fotowareenableexport==="True") {
                            this.fwWin.location.href = this.fotowarepath +
                                "ClientResources/plugin/fotoware.html?fotowareBaseUrl=" +
                                this.fotowaretenanturl +
                                "/fotoweb/widgets/publish&i=" +
                                event.data.asset.href +
                                "&" + Math.random() +
                                "&cn=" + this.fotowarecookiename +
                                "&pass=true&" + this.fotowareextraquerystring;
                            return;
                        } else {
                            //get the default image
                            this.fwWin.close();

                            this.assetChoosen(event.data.asset.href);

                            //remove listener
                            window.removeEventListener("message", this._listener, true);
                            this.listenerBound = false;
                        }

                    }
                    else if (event.data.event === 'assetExported') {
                        this.fwWin.close();
                        this.assetChoosenExport(event.data.export.export.source.replace(this.fotowaretenanturl, ""), event.data.export.export.image.normal.replace(this.fotowaretenanturl, ""))

                        //remove listener
                        window.removeEventListener("message", this._listener, true);
                        this.listenerBound = false;

                    }
                    //there are more events sent, like frame load, page load, focus
                    
                }
            },
            assetChoosen: function (assethref){
                var self = this;
                when(self.getFetchCf(
                        self.fotowaremodulepath + "authentication/fetchcf",
                        {
                            hostcontent: "",
                            image: assethref
                        }),
                    function (data) {
                        self._setValueAndFireOnChange(data + "");
                    });
            },
            assetChoosenExport: function (assethref, exporturl) {
                var self = this;
                when(self.getFetchCf(
                        self.fotowaremodulepath + "authentication/fetchcf",
                        {
                            hostcontent: "",
                            image: assethref,
                            exporturl: exporturl
                        }),
                    function (data) {
                        self._setValueAndFireOnChange(data + "");
                    });
            },
            //if you want to get other thumbnail from fotoware for types that episerver do not have thumbnails
            //_updateDisplayNode: function (content) {
            //    this.inherited(arguments);
            //    if (content) {
            //        this.thumbnail.src = this._getThumbnailUrl(content).toString();
            //        //custom thumbnail
            //        if (!content.name.match(/.(jpg|jpeg|png|gif|bmp|jpe)$/i)) {
            //            this.thumbnail.src = this.fotowaremodulepath + "authentication/GenerateThumb?contentid=" + content.contentLink;
            //            this.thumbnail.style = "object-fit:contain;";
            //        }
            //    }
            //    this.stateNode.title = content ? content.name : "";
            //},
            //buildUrl: function (url, assethref) {

            //    var baseUrl = url + '?i=' + assethref;

            //    baseUrl = baseUrl + this.getUrlPart('&w=', this.defaultWidth);
            //    baseUrl = baseUrl + this.getUrlPart('&h=', this.defaultHeight);
            //    baseUrl = baseUrl + this.getUrlPart('&p=', this.defaultPreset);

            //    return baseUrl;
            //},
            getUrlPart: function (key, value) {
                if (typeof (value) != 'undefined' && value != "") {
                    return key + value;
                }
                return "";
            },

            _setValueAttr: function (value) {
                if (this.disableMediaSelector==="True" && this.loaded && value != null) {
                    this._dialogService.alert(this.uploadDisabledMessage);
                    return;
                }
                this.inherited(arguments);
            },

            _onButtonClick: function () {
                if (this.readOnly) {
                    return;
                }

                if (this.disableMediaSelector==="True") {
                    this._onFotoWareButtonClick();
                    return;
                }


                this.inherited(arguments);
            }
        });
    });
