﻿using EPiServer;
using EPiServer.Core;
using FotoWare.PlugIns.Episerver;
using FotoWare.PlugIns.Episerver.Interfaces;
using System;
using System.Linq;

namespace AlloyTemplates.FotoWare
{
    public class ArchiveFolderResolver : IFolderResolver
    {
        private readonly IFotoWareSettings _fotowareSettings;
        private readonly IContentRepository _contentRepository;

        public ArchiveFolderResolver(IFotoWareSettings fotowareSettings, IContentRepository contentRepository)
        {
            _fotowareSettings = fotowareSettings;
            _contentRepository = contentRepository;
        }

        public ContentReference GetAssetFolder(AssetInformation assetInformation, FotoWareItemMetadata dataItem)
        {
            if (_fotowareSettings.RootFolderId == null)
            {
                throw new NullReferenceException("You haven't specified FotoWareSettings for RootFolderId, please implement or set FotoWare:RootFolderId in appsettings web.config.");
            }

            if (dataItem.Filename == null)//only integration test at first request //todo change this
            {
                return _fotowareSettings.RootFolderId;
            }

            var metadata = dataItem.MetadataAsJson["metadata"];
            string mainCategory = null;
            if (metadata.ContainsKey("700"))//todo change 700 to a data that exists in your fotoware implementation
            {
                mainCategory = metadata["700"]["value"];
            }
            else
            {
                return _fotowareSettings.RootFolderId;
            }

            var childFolders = _contentRepository.GetChildren<ContentFolder>(_fotowareSettings.RootFolderId);

            var categoryFolder = childFolders.FirstOrDefault(f => String.Equals(f.Name, mainCategory, StringComparison.OrdinalIgnoreCase));

            if (categoryFolder == null)
            {
                var newFolder = _contentRepository.GetDefault<ContentFolder>(_fotowareSettings.RootFolderId);
                newFolder.Name = mainCategory;
                return _contentRepository.Save(newFolder);
            }

            return categoryFolder.ContentLink;
        }
    }
}